@@ -22,7 +22,7 @@ gtk.gdk.threads_init()
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.0", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.1", False)
 except ImportError:
     pass
 
@@ -70,6 +70,36 @@ def create_config_dir():
     return path
 
 
+class DiscoveryThread(Thread):
+
+
+    def __init__(self, url):
+        Thread.__init__(self)
+        self.url = url
+
+
+    def run(self):
+        try:
+            gobject.idle_add(self.done, feedcircuit.discover_feed(self.url), None)
+        except Exception, ex:
+            gobject.idle_add(self.done, None, ex)
+
+
+    def done(self, result, ex):
+        self.result = result
+        self.exception = ex
+        gtk.main_quit()
+
+    def discover(self):
+        self.result = None
+        self.exception = None
+        self.start()
+        gtk.main()
+        if self.exception:
+            raise self.exception
+        return self.result
+
+
 class ConfigParserEx(ConfigParser.SafeConfigParser):
 
 
@@ -396,13 +426,12 @@ class EditWindow(gtk.Dialog):
         adv_window.ignore_print_version_button.set_active(self.ignore_print_version)
         adv_window.include_entry.set_text(self.include)
         adv_window.exclude_entry.set_text(self.exclude)
-        result = adv_window.run()
-        adv_window.hide()
-        if result == gtk.RESPONSE_ACCEPT:
+        if adv_window.run() == gtk.RESPONSE_ACCEPT:
             self.days_to_keep = adv_window.days_editor.get_value()
             self.ignore_print_version = adv_window.ignore_print_version_button.get_active()
             self.include = adv_window.include_entry.get_text()
             self.exclude = adv_window.exclude_entry.get_text()
+        adv_window.destroy()
 
 
 class Program(hildon.Program):
@@ -426,15 +455,16 @@ class Program(hildon.Program):
         menu = gtk.Menu()
         self.__toolbar = gtk.Toolbar()
         self.__toolbar.set_style(gtk.TOOLBAR_ICONS)
-        self.append_action(menu, self.__toolbar, _("Open"), gtk.STOCK_OPEN, self.open)
-        self.append_action(menu, self.__toolbar, _("Refresh all"), gtk.STOCK_REFRESH, self.refresh)
-        self.append_action(menu, self.__toolbar, _("Add..."), gtk.STOCK_ADD, self.add)
-        self.append_action(menu, self.__toolbar, _("Edit..."), None, self.edit)
-        self.append_action(menu, self.__toolbar, _("Refresh"), None, self.refresh_current)
-        self.append_action(menu, self.__toolbar, _("Remove"), None, self.remove)
+        self.append_action(menu, self.__toolbar, None, gtk.STOCK_REFRESH, self.refresh)
+        self.append_action(menu, self.__toolbar, None, gtk.STOCK_ADD, self.add)
         menu.append(gtk.SeparatorMenuItem())
-        self.append_action(menu, self.__toolbar, _("About..."), None, self.about)
-        self.append_action(menu, self.__toolbar, _("Close"), None, self.stop)
+        self.append_action(menu, None, _("Refresh selected"), None, self.refresh_current)
+        self.append_action(menu, None, None, gtk.STOCK_OPEN, self.open)
+        self.append_action(menu, None, None, gtk.STOCK_EDIT, self.edit)
+        self.append_action(menu, None, None, gtk.STOCK_DELETE, self.remove)
+        menu.append(gtk.SeparatorMenuItem())
+        self.append_action(menu, None, None, gtk.STOCK_ABOUT, self.about)
+        self.append_action(menu, None, None, gtk.STOCK_QUIT, self.stop)
         self.__progress_label = gtk.Label()
         self.__progress_label.set_alignment(0, 0.5)
         self.__progress_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
@@ -474,10 +504,13 @@ class Program(hildon.Program):
 
 
     def append_action(self, menu, toolbar, title, stock, callback):
-        item = gtk.MenuItem(title)
+        if stock:
+            item = gtk.ImageMenuItem(stock)
+        else:
+            item = gtk.MenuItem(title)
         item.connect("activate", callback)
         menu.append(item)
-        if stock:
+        if toolbar:
             button = gtk.ToolButton(stock)
             button.connect("clicked", callback)
             toolbar.insert(button, -1)
@@ -518,7 +551,7 @@ class Program(hildon.Program):
                 type=gtk.MESSAGE_WARNING, buttons = gtk.BUTTONS_OK, 
                 message_format = _("The feed is empty"))
             msg_dlg.run()
-            msg_dlg.hide()
+            msg_dlg.destroy()
 
 
     def get_selected_feeds(self):
@@ -615,28 +648,31 @@ class Program(hildon.Program):
         if len(feed_configs):
             feed_config = feed_configs[0]
             edit_window = EditWindow(self.__window, feed_config)
-            result = edit_window.run()
-            edit_window.hide()
-            if result == gtk.RESPONSE_ACCEPT:
+            if edit_window.run() == gtk.RESPONSE_ACCEPT:
                 edit_window.update_feed()
                 self.__model.update(feed_config.id)
+            edit_window.destroy()
 
 
     def add(self, source):
         add_window = AddWindow(self.__window)
-        result = add_window.run()
+        response = add_window.run()
         add_window.hide()
-        if result == gtk.RESPONSE_ACCEPT:
+        if response == gtk.RESPONSE_ACCEPT:
             url = add_window.url_entry.get_text()
             title = ""
+            banner = hildon.hildon_banner_show_animation(self.__window, "qgn_indi_pball_a", _("Discovering feed"))
             try:
-                (url, title, icon_url) = feedcircuit.discover_feed(url)
+                discovery_thread = DiscoveryThread(url)
+                (url, title, icon_url) = discovery_thread.discover()
             except Exception, ex:
                 msg_dlg = gtk.MessageDialog(parent = self.__window, flags = gtk.DIALOG_MODAL,
                     type=gtk.MESSAGE_WARNING, buttons = gtk.BUTTONS_OK, 
                     message_format = _("Cannot discover feed at given url") + "\n" + str(ex))
                 msg_dlg.run()
-                msg_dlg.hide()
+                msg_dlg.destroy()
+            finally:
+                banner.destroy()
             icon_filename = ""
             try:
                 if icon_url:
@@ -646,33 +682,40 @@ class Program(hildon.Program):
                 pass
             feed_config =FeedConfig(url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0)
             edit_window = EditWindow(self.__window, feed_config)
-            result = edit_window.run()
-            edit_window.hide()
-            if result == gtk.RESPONSE_ACCEPT:
+            if edit_window.run() == gtk.RESPONSE_ACCEPT:
                 edit_window.update_feed()
                 self.__model.append(feed_config)
             elif icon_filename:
-                os.remove(icon_filename)
+                os.remove(os.path.join(create_config_dir(), icon_filename))
+            edit_window.destroy()
+        add_window.destroy()
 
 
 
     def remove(self, source):
-        for feed_config in list(self.get_selected_feeds()):
-            feed = feedcircuit.Feed(None, path = self.__config_path,
-                cache_path = ".cache", filename = feed_config.id + ".html")
-            feed.delete()
-            if feed_config.icon_filename:
-                icon_filepath = os.path.join(self.__config_path, feed_config.icon_filename)
-                if os.path.exists(icon_filepath):
-                    os.remove(icon_filepath)
-            self.__model.delete(feed_config.id)
+        selected = list(self.get_selected_feeds())
+        if len(selected):
+            dialog = gtk.MessageDialog(self.__window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, _("Are you sure you want to delete selected feeds?"))
+            response = dialog.run()
+            dialog.destroy()
+            if response == gtk.RESPONSE_YES:
+                for feed_config in selected:
+                    feed = feedcircuit.Feed(None, path = self.__config_path,
+                        cache_path = ".cache", filename = feed_config.id + ".html")
+                    feed.delete()
+                    if feed_config.icon_filename:
+                        icon_filepath = os.path.join(self.__config_path, feed_config.icon_filename)
+                        if os.path.exists(icon_filepath):
+                            os.remove(icon_filepath)
+                    self.__model.delete(feed_config.id)
 
 
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.0")
+        dialog.set_version("0.6.1")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
-        dialog.hide()
+        dialog.destroy()
